
## language_tui.py: text mode language selection dialog
## Copyright (C) 2002-2014 Red Hat, Inc.
## Copyright (C) 2002, 2003 Brent Fox <bfox@redhat.com>
## Copyright (C) 2013-2014 Parag Nemade <pnemade@redhat.com>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import snack
import locale
import language_backend
import install_packages
import subprocess
import gettext
import string

locale.setlocale(locale.LC_ALL, "")
_ = lambda x: gettext.ldgettext("system-config-language", x)
N_ = lambda x: x
gettext.bind_textdomain_codeset("system-config-language", locale.nl_langinfo(locale.CODESET))

languageBackend = language_backend.LanguageBackend()

installed_list = []
list_to_install = []

langlist_with_cc = ['zh_CN', 'zh_TW', 'pt_BR']

class LanguageWindow:
    def __init__(self):
        self.langDict = {}

    def DrawUI(self, screen):
        defaultLang, self.installedLangs = languageBackend.getInstalledLangs()

        bb = snack.ButtonBar(screen, ["Yes", "No"])
        textBox = snack.TextboxReflowed(40,
                _("Select the language for the system."))

        llist = self.populateListBox(defaultLang)
        g = snack.GridFormHelp(screen, _("Language Selection"), "kbdtype", 1, 4)
        g.add(textBox, 0, 0)
        g.add(llist, 0, 1, padding=(0, 1, 0, 1))
        g.add(bb, 0, 3, growx=1)
        rc = g.runOnce()

        button = bb.buttonPressed(rc)
        if button == "no":
            return -1

        choice = llist.current()
        defaultLang, sysfontacm, sysfont = self.langDict[choice]

        langid = languageBackend.removeEncoding(defaultLang)
        install_packages.installed_packages()
        if langid not in langlist_with_cc:
            langcode = langid.split("_")[0]
        else:
            langcode = langid

        installed = install_packages.check_if_already_installed(langcode)
        if installed is "no":
            screen.finish()
            cstr = _(" language support is not installed, do you want to install it?")
            cstr = langid + cstr
            cscreen = snack.SnackScreen()
            bbc = snack.ButtonBar(cscreen, ["OK", "Cancel"])
            textBoxc = snack.TextboxReflowed(40, cstr)
            gc = snack.GridFormHelp(screen, _("Installing Confirm"), "kbdtype", 1, 4)
            gc.add(textBoxc, 0, 0)
            gc.add(bbc, 0, 3, growx=1)

            rcc = gc.runOnce()
            buttonc = bbc.buttonPressed(rcc)

            if buttonc == "ok":
                cscreen.finish()
                pkg_list = install_packages.get_language_packages(langcode)

                if not pkg_list:
                    imsgx = _("No package support is available for language ")
                    imsgx = imsgx + langid
                    screenx = snack.SnackScreen()
                    bbx = snack.ButtonBar(screenx, ["OK"])
                    textBoxx = snack.TextboxReflowed(40, imsgx)
                    gx = snack.GridFormHelp(screenx, _("No packages available"), "kbdtype", 1, 4)
                    gx.add(textBoxx, 0, 0)
                    gx.add(bbx, 0, 3, growx=1)

                    rcx = gx.runOnce()
                    buttonx = bbx.buttonPressed(rcx)
                    if buttonx == "ok":
                        screenx.finish()
                else:
                    for pkgs in install_packages.uniq_installed_pkgs:
                        installed_list.append(pkgs)
                    for name in pkg_list:
                        if name not in installed_list:
                            list_to_install.append(name)

                    cmd = "pkcon install "
                    if list_to_install:
                        for item in list_to_install:
                            cmd = cmd + " {0}".format(item)
                        output = subprocess.check_output(cmd, shell=True)
                        imsg1 = _(" language support is now installed. Setting it as the default system language.")
                        imsg1 = langid + imsg1
                        screen1 = snack.SnackScreen()
                        bb1 = snack.ButtonBar(screen1, ["OK"])
                        textBox1 = snack.TextboxReflowed(40, imsg1)
                        g1 = snack.GridFormHelp(screen1, _("Language installed"), "kbdtype", 1, 4)
                        g1.add(textBox1, 0, 0)
                        g1.add(bb1, 0, 3, growx=1)

                        rc1 = g1.runOnce()
                        button1 = bb1.buttonPressed(rc1)
                        if button1 == "ok":
                            screen1.finish()
                        install_packages.add_langpack_to_installed_list(langcode)
                    else:
                        imsg2 = _(" language support packages are already installed, Therefore setting only the default system language.")
                        imsg2 = langid + imsg2
                        screen2 = snack.SnackScreen()
                        bb2 = snack.ButtonBar(screen2, ["OK"])
                        textBox2 = snack.TextboxReflowed(40, imsg2)
                        g2 = snack.GridFormHelp(screen2, _("Language packages already installed"), "kbdtype", 1, 4)
                        g2.add(textBox2, 0, 0)
                        g2.add(bb2, 0, 3, growx=1)
                        rc2 = g2.runOnce()
                        button2 = bb2.buttonPressed(rc2)
                        if button2 == "ok":
                            screen2.finish()
                        install_packages.add_langpack_to_installed_list(langcode)
        else:
            imsg3 = _(" language support is already installed, Therefore setting only the default system language.")
            imsg3 = langid + imsg3
            screen3 = snack.SnackScreen()
            bb3 = snack.ButtonBar(screen3, ["OK"])
            textBox3 = snack.TextboxReflowed(40, imsg3)
            g3 = snack.GridFormHelp(screen3, _("Language already installed"), "kbdtype", 1, 4)
            g3.add(textBox3, 0, 0)
            g3.add(bb3, 0, 3, growx=1)

            rc3 = g3.runOnce()
            button3 = bb3.buttonPressed(rc3)
            if button3 == "ok":
                screen3.finish()

        if self.installedLangs == None:
            languageBackend.writeI18N(defaultLang, "", sysfont, sysfontacm)
        else:
            modules = self.installedLangs[0]
            for lang in self.installedLangs[1:]:
                modules = modules + ":" + lang

            languageBackend.writeI18N(defaultLang, modules, sysfont, sysfontacm)

    def populateListBox(self, defaultLang):
        llist = snack.Listbox(8, scroll=1, returnExit=0)
        lines = languageBackend.readTable()

        if defaultLang == None:
            llist.append('English (USA)', 'en_US.UTF-8')
            self.installedLangs = ['en_US.UTF-8:en']
            llist.setCurrent("en_US.UTF-8")
            return llist


        default = ""
        for line in lines:
            tokens = string.split(line)

            if self.installedLangs == None:
                langBase = languageBackend.removeEncoding(tokens[0])
                name = ""
                for token in tokens[3:]:
                    name = name + " " + token

                    if languageBackend.removeEncoding(defaultLang) == langBase:
                        default = tokens[0]

                llist.append(name, tokens[0])
                self.langDict[tokens[0]] = (tokens[0], tokens[1], tokens[2])

            else:
                if '.' in tokens[0]:
                    #Chop encoding off so we can compare to self.installedLangs
                    langBase = languageBackend.removeEncoding(tokens[0])
                    if langBase in self.installedLangs:
                        name = ""
                        for token in tokens[3:]:
                            name = name + " " + token

                        if languageBackend.removeEncoding(defaultLang) == langBase:
                            default = tokens[0]

                        llist.append(name, tokens[0])
                        self.langDict[tokens[0]] = (tokens[0], tokens[1], tokens[2])

        llist.setCurrent(default)
        return llist

class childWindow:
    def __init__(self):
        screen = snack.SnackScreen()
        screen.drawRootText(0, 0, "system-config-language - (C) 2004-2014 Red Hat, Inc.")
        lw = LanguageWindow()

        DONE = 0
        while not DONE:
            rc = lw.DrawUI(screen)
            if rc == -1:
                screen.finish()
                DONE = 1
            else:
                screen.finish()
                self.runConfig(rc)
                DONE = 1

    def runConfig(self, rc):
        pass
